"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DockerClient = void 0;
const dockerode_1 = __importDefault(require("dockerode"));
const path_1 = __importDefault(require("path"));
const ts_log_1 = require("ts-log");
class DockerClient {
    constructor(store, options) {
        this.docker = new dockerode_1.default();
        this.logger = options.logger || ts_log_1.dummyLogger;
        this.store = store;
        this.utilityImage = options.utilityImage || 'alpine:3.12.0';
    }
    ensureUtilityImageAvailable() {
        return __awaiter(this, void 0, void 0, function* () {
            if ((yield this.hasImage(this.utilityImage)) === false) {
                this.logger.info(`Pulling ${this.utilityImage}`);
                yield this.pullPromise(this.utilityImage);
                const image = yield this.getImage(this.utilityImage);
                const managedImages = this.store.get('managedImages');
                const newValue = [image.Id];
                if (managedImages !== undefined) {
                    newValue.concat(managedImages);
                }
                yield this.store.set('managedImages', newValue);
            }
        });
    }
    makeSnapshot(services) {
        return __awaiter(this, void 0, void 0, function* () {
            const snapshot = yield this.store.initSnapshot();
            yield this.ensureUtilityImageAvailable();
            for (const service of [...services.values()]) {
                this.logger.info(`Snapshotting service ${service.name}`);
                for (const volume of service.volumes) {
                    yield this.docker.run(this.utilityImage, ['sh', '-c', `cd ${service.dataPath} && tar cvf /snapshot/${volume}.tar .`], process.stdout, {
                        HostConfig: {
                            AutoRemove: true,
                            Binds: [
                                `${snapshot}:/snapshot`,
                                `${volume}:${service.dataPath}`
                            ]
                        }
                    });
                }
            }
        });
    }
    restoreSnapshot(snapshot, services) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.canRestore(snapshot);
            yield this.ensureUtilityImageAvailable();
            for (const service of services) {
                this.logger.info(`Restoring service ${service.name}`);
                for (const volume of service.volumes) {
                    const snapshotPath = path_1.default.join(this.store.snapshotsDir, snapshot, `${volume}.tar`);
                    yield this.docker.run(this.utilityImage, ['sh', '-c', 'cd /restore && tar xvf /snapshot.tar'], process.stdout, {
                        HostConfig: {
                            AutoRemove: true,
                            Binds: [
                                `${volume}:/restore`,
                                `${snapshotPath}:/snapshot.tar`
                            ]
                        }
                    });
                }
            }
        });
    }
    createVolume(name) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.info(`Creating volume ${name}...`);
            yield this.docker.createVolume({
                Name: name
            });
        });
    }
    removeVolume(name) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.info(`Removing volume ${name}...`);
            yield this.docker.getVolume(name).remove();
        });
    }
    cleanup() {
        return __awaiter(this, void 0, void 0, function* () {
            const managedImageIds = this.store.get('managedImages');
            if (managedImageIds === undefined) {
                this.logger.debug('No managed images to remove');
                return;
            }
            for (const imageId of managedImageIds) {
                this.logger.info(`Removing managed image ${imageId}...`);
                const image = this.docker.getImage(imageId);
                this.logger.debug(image);
                yield image.remove();
            }
        });
    }
    getImage(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const images = yield this.docker.listImages();
            for (const image of images) {
                if (image.RepoTags && image.RepoTags.indexOf(name) !== -1) {
                    return image;
                }
            }
        });
    }
    hasImage(name) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.getImage(name)) !== undefined;
        });
    }
    canRestore(snapshot) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.store.validateSnapshot(snapshot);
        });
    }
    pullPromise(imageName) {
        return new Promise((resolve, reject) => {
            this.docker.pull(imageName, {}, (error, stream) => {
                if (error)
                    return reject(error);
                const onFinished = (error, output) => {
                    if (error)
                        return reject(error);
                    resolve(output);
                };
                const onProgress = (event) => {
                    if (event.progress) {
                        this.logger.info(event.progress);
                    }
                };
                this.docker.modem.followProgress(stream, onFinished, onProgress);
            });
        });
    }
}
exports.DockerClient = DockerClient;
//# sourceMappingURL=DockerClient.js.map