"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DockerComposeStack = void 0;
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const docker_compose_1 = require("docker-compose");
const _1 = require("./");
const ts_log_1 = require("ts-log");
var StackState;
(function (StackState) {
    StackState["up"] = "up";
    StackState["down"] = "down";
    StackState["stopped"] = "stopped";
})(StackState || (StackState = {}));
class DockerComposeStack {
    constructor(store, dockerClient, options) {
        this.docker = dockerClient;
        this.logger = options.logger || ts_log_1.dummyLogger;
        this.projectName = options.projectName || 'cardano-graphql';
        this.store = store;
        this.statefulServices = options.statefulServices || new Map([
            ['postgres', _1.postgresService(this.projectName)],
            ['cardano-node', _1.cardanoNodeService(this.projectName)]
        ]);
    }
    init(workingDirectory) {
        return __awaiter(this, void 0, void 0, function* () {
            this.store.set('workingDirectory', workingDirectory);
            const compose = yield this.store.get('compose');
            if (compose && compose.createdAt !== undefined) {
                this.logger.info('Already initialized');
                return;
            }
            yield fs_extra_1.default.copyFile(path_1.default.resolve(__dirname, 'docker-compose.yml'), this.store.dockerComposeFilePath);
            yield this.store.set('compose', { createdAt: Date.now() });
            this.logger.info('docker-compose.yml created');
        });
    }
    up() {
        return __awaiter(this, void 0, void 0, function* () {
            const currentState = yield this.currentState();
            if (currentState === StackState.up)
                return;
            this.logger.info('docker-compose up');
            return docker_compose_1.upAll({
                composeOptions: ['-p', this.projectName]
            });
        });
    }
    down() {
        return __awaiter(this, void 0, void 0, function* () {
            const currentState = yield this.currentState();
            if (currentState === StackState.down)
                return;
            this.logger.info('docker-compose down');
            yield docker_compose_1.down({
                composeOptions: ['-p', this.projectName]
            });
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            const currentState = yield this.currentState();
            if (currentState === StackState.stopped || currentState === StackState.down)
                return;
            this.logger.info('docker-compose stop');
            yield docker_compose_1.stop({
                composeOptions: ['-p', this.projectName]
            });
        });
    }
    makeSnapshot() {
        return __awaiter(this, void 0, void 0, function* () {
            const beforeState = yield this.currentState();
            yield this.stop();
            try {
                yield this.docker.makeSnapshot(this.statefulServices);
            }
            catch (error) {
                this.logger.error(error);
            }
            finally {
                if (beforeState === StackState.up) {
                    yield this.up();
                }
                this.logger.info('Snapshot complete');
            }
        });
    }
    restoreSnapshot(snapshotName) {
        return __awaiter(this, void 0, void 0, function* () {
            const snapshot = snapshotName !== undefined ? snapshotName : (yield this.store.snapshotsList())[0];
            const allServices = [...this.statefulServices.values()];
            yield this.docker.canRestore(snapshot);
            const beforeState = yield this.currentState();
            yield this.down();
            yield this.removeServiceVolumes(allServices);
            yield this.createServiceVolumes(allServices);
            yield this.docker.restoreSnapshot(snapshot, allServices);
            if (beforeState === StackState.up) {
                yield this.up();
            }
            this.logger.info('Snapshot restore complete');
        });
    }
    rebuildService(services, backup = true) {
        return __awaiter(this, void 0, void 0, function* () {
            if (backup === true) {
                yield this.makeSnapshot();
            }
            const beforeState = yield this.currentState();
            yield this.down();
            try {
                yield this.removeServiceVolumes(services);
            }
            catch (error) {
                this.logger.error(error);
            }
            finally {
                if (beforeState === StackState.up) {
                    yield this.up();
                }
            }
        });
    }
    currentState() {
        return __awaiter(this, void 0, void 0, function* () {
            const psResult = yield docker_compose_1.ps({
                commandOptions: ['-q']
            });
            if (psResult.out.length === 0) {
                const stoppedContainers = yield docker_compose_1.ps({
                    commandOptions: ['-aq']
                });
                if (stoppedContainers.out.length === 0) {
                    return StackState.down;
                }
                else {
                    return StackState.stopped;
                }
            }
            else {
                return StackState.up;
            }
        });
    }
    createServiceVolumes(services) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const service of services) {
                for (const volume of service.volumes) {
                    yield this.docker.createVolume(volume);
                }
            }
        });
    }
    removeServiceVolumes(services) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                for (const service of services) {
                    for (const volume of service.volumes) {
                        yield this.docker.removeVolume(volume);
                    }
                }
            }
            catch (error) {
                this.logger.warn(error.message);
            }
        });
    }
}
exports.DockerComposeStack = DockerComposeStack;
//# sourceMappingURL=DockerComposeStack.js.map